/**
 * Copyright (c) 2013, impossibl.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *  * Neither the name of impossibl.com nor the names of its contributors may
 *    be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package com.impossibl.postgres.system.procs;

import com.impossibl.postgres.api.data.ACLItem;
import com.impossibl.postgres.system.Context;
import com.impossibl.postgres.system.ConversionException;
import com.impossibl.postgres.types.Type;

import java.io.IOException;
import java.text.ParseException;

public class ACLItems extends SimpleProcProvider {

  public ACLItems() {
    super(new TxtEncoder(), new TxtDecoder(), null, null, "aclitem");
  }

  static class TxtDecoder extends AutoConvertingTextDecoder<ACLItem> {

    TxtDecoder() {
      super(ACLItem::toString);
    }

    @Override
    public Class<ACLItem> getDefaultClass() {
      return ACLItem.class;
    }

    @Override
    protected ACLItem decodeNativeValue(Context context, Type type, Short typeLength, Integer typeModifier, CharSequence buffer, Class<?> targetClass, Object targetContext) throws IOException, ParseException {
      return ACLItem.parse(buffer.toString());
    }

  }

  static class TxtEncoder extends AutoConvertingTextEncoder<ACLItem> {

    private static ACLItem parse(String value) throws ConversionException {
      try {
        ACLItem item = ACLItem.parse(value);
        if (item == null) {
          throw new ConversionException("Invalid ACLItem");
        }
        return item;
      }
      catch (ParseException e) {
        throw new ConversionException(e);
      }
    }

    TxtEncoder() {
      super(TxtEncoder::parse);
    }

    @Override
    protected Class<ACLItem> getDefaultClass() {
      return ACLItem.class;
    }

    @Override
    protected void encodeNativeValue(Context context, Type type, ACLItem value, Object sourceContext, StringBuilder buffer) throws IOException {
      buffer.append(value.toString());
    }

  }

}
